# Stubs for django.shortcuts (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from django.db import models

from typing import Any, Optional, Union, TypeVar, Generic, Type
from typing_extensions import Protocol

T = TypeVar('T', bound=models.Model)

class SupportsAbsoluteURL(Protocol):
    def get_absolute_url(self) -> str:
       ...

from django.http import HttpResponsePermanentRedirect, HttpResponseRedirect

# def render_to_response(template_name: Any, context: Optional[Any] = ..., content_type: Optional[Any] = ..., status: Optional[Any] = ..., using: Optional[Any] = ...): ...
# def render(request: Any, template_name: Any, context: Optional[Any] = ..., content_type: Optional[Any] = ..., status: Optional[Any] = ..., using: Optional[Any] = ...): ...
def redirect(to: Union[str, SupportsAbsoluteURL], *, permanent: bool = False) -> Union[HttpResponseRedirect, HttpResponsePermanentRedirect]: ... # *args: Any, permanent: bool = ..., **kwargs: Any): ...
def get_object_or_404(klass: Type[T], *args: Any, **kwargs: Any) -> T: ...
# def get_list_or_404(klass: Any, *args: Any, **kwargs: Any): ...
# def resolve_url(to: Any, *args: Any, **kwargs: Any): ...
