# Generated by Django 2.1b1 on 2018-07-10 17:42

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('testing', '0001_initial'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='Widget',
            new_name='Trinket',
        ),
    ]
