from typing import Any

class Field:
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...

class CharField(Field): ...

class ChoiceField(Field): ...

class MultipleChoiceField(Field): ...

class ModelChoiceField(Field): ...

class ModelMultipleChoiceField(Field): ...

class FileField(Field): ...

class EmailField(Field): ...
